----------------------------------------------------------------------------------
-- PS2 controller test bench

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity PS2_controller_tb is
--  Port ( );
end PS2_controller_tb;

architecture Behavioral of PS2_controller_tb is

    constant clock_period : time := 10 ns;
    constant PS2_CLK_period : time := 2*3.14 * clock_period;
     
    -- Simulation data for key ENTER: 0x5A = 0101 1010
    -- STOP PAR B7 ... B1 B0 START
    --  1    1  0  ... 1  0    0
    constant data_sim : std_logic_vector(10 downto 0) := "11010110100";
    signal CLK100MHZ : std_logic := '1';
    signal CPU_RESETN, PS2_CLK, PS2_DATA, eot : std_logic;
    signal data : std_logic_vector(8 downto 0);

begin

    UUT: entity work.PS2Controller(Behavioral)
    port map(
        CLK100MHZ => CLK100MHZ,
        CPU_RESETN => CPU_RESETN,
        PS2_CLK => PS2_CLK,
        PS2_DATA => PS2_DATA,
        data => data,
        eot => eot
    );

    stimulus_clk: process 
    begin 
        CLK100MHZ <= not CLK100MHZ;
        wait for clock_period/2;
    end process;

    stimuli_oth: process 
    begin 
        
        -- Long reset 
        CPU_RESETN <= '0'; -- active low
        PS2_CLK   <= '1'; -- idle 
        PS2_DATA <= '1';
        wait for clock_period*3; 
        
        -- Remove reset, idle state 
        CPU_RESETN <= '1';        
        wait for clock_period*3;
        
        -- let's delay the clock to desynchonize 
        wait for clock_period*0.3334;
        
        for i in 0 to 10 loop
            PS2_DATA <= data_sim(i);
            wait for PS2_CLK_period/8;
            PS2_CLK <= '0';
            wait for PS2_CLK_period/2;
            PS2_CLK <= '1';
            wait for PS2_CLK_period/2;
        end loop;
        
        -- Wait forever 
        wait;
    
    end process;

end Behavioral;
