----------------------------------------------------------------------------------
-- Control unit - FSM
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity control_unit is
    port (
        clock : in STD_LOGIC;
        reset : in STD_LOGIC;
        pulse : in STD_LOGIC;
        PS2_DATA_S : in STD_LOGIC;
        eot : out STD_LOGIC;
        we : out STD_LOGIC);
end control_unit;

architecture Behavioral of control_unit is

    --Use descriptive names for the states, like st1_reset, st2_search
    type state_type is (ST_IDLE, ST_START, ST_B0, ST_B1, ST_B2, ST_B3,
                        ST_B4, ST_B5, ST_B6, ST_B7, ST_PAR, ST_STOP);
    signal state, next_state : state_type;
    --Declare internal signals for all outputs of the state-machine
    signal we_i, eot_i : std_logic;

begin

SYNC_PROC: process (clock)
begin
    if rising_edge(clock) then
        if (reset = '1') then
            state <= ST_IDLE;
            eot <= '0';
            we <= '0';
        else
            state <= next_state;
            eot <= eot_i;
            we <= we_i;
        end if;
    end if;
end process;

--MEALY State-Machine - Outputs based on state and inputs
OUTPUT_DECODE: process (state, pulse, PS2_DATA_S)
begin
    -- default 
    we_i <= '0';
    eot_i <= '0';
    
    -- cover only the cases when outputs are different than default
    case state is
        when ST_START | ST_B0 | ST_B1 | ST_B2 | ST_B3 | 
             ST_B4 | ST_B5 | ST_B6 | ST_B7 =>
            if pulse = '1' then
                we_i <= '1';                
            end if;
        when ST_PAR => 
            if pulse = '1' and PS2_DATA_S = '1' then
                eot_i <= '1';
            end if;
        when others => 
            we_i <= '0';
            eot_i <= '0';        
    end case;
end process;

NEXT_STATE_DECODE: process (state, pulse, PS2_DATA_S)
   begin
      --declare default state for next_state to avoid latches
      next_state <= state;  --default is to stay in current state
      
      case (state) is
         when ST_IDLE =>
            if pulse = '1' and PS2_DATA_S = '0' then
               next_state <= ST_START;
            end if;
         when ST_START =>
            if pulse = '1' then next_state <= ST_B0; end if;
         when ST_B0 =>
            if pulse = '1' then next_state <= ST_B1; end if;
         when ST_B1 =>
            if pulse = '1' then next_state <= ST_B2; end if;
         when ST_B2 =>
            if pulse = '1' then next_state <= ST_B3; end if;
         when ST_B3 =>
            if pulse = '1' then next_state <= ST_B4; end if;
         when ST_B4 =>
            if pulse = '1' then next_state <= ST_B5; end if;
         when ST_B5 =>
            if pulse = '1' then next_state <= ST_B6; end if;   
         when ST_B6 =>
            if pulse = '1' then next_state <= ST_B7; end if; 
         when ST_B7 =>
            if pulse = '1' then next_state <= ST_PAR; end if; 
         when ST_PAR =>
            if pulse = '1' then next_state <= ST_STOP; end if;    
         when ST_STOP =>
            next_state <= ST_IDLE;             
         when others =>
            next_state <= ST_IDLE;
      end case;
   end process;

end Behavioral;
