----------------------------------------------------------------------------------
-- Negative edge detector, a.k.a. pulse generator 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity pulse_gen is
    port (
        clock : in STD_LOGIC;
        reset : in STD_LOGIC;
        PS2_CLK_S : in STD_LOGIC;
        pulse : out STD_LOGIC);
end pulse_gen;

architecture Behavioral of pulse_gen is

    --Use descriptive names for the states, like st1_reset, st2_search
    type state_type is (ST_IDLE, ST_ACTIVE, ST_WAIT);
    signal state, next_state : state_type;

    --Declare internal signals for all outputs of the state-machine
    signal pulse_i : std_logic;

begin
    --Insert the following in the architecture after the begin keyword
    SYNC_PROC: process (clock)
    begin
        if rising_edge(clock) then
            if reset = '1' then
                state <= ST_IDLE;
                pulse <= '0';
            else
                state <= next_state;
                pulse <= pulse_i;
            end if;
        end if;
    end process;

    --MOORE State-Machine - Outputs based on state only
    OUTPUT_DECODE: process (state)
    begin
        if state = ST_IDLE then
            pulse_i <= '0';
        elsif state = ST_ACTIVE then
            pulse_i <= '1';
        elsif state = ST_WAIT then
            pulse_i <= '0';
        else
            pulse_i <= '0';
        end if;
    end process;

    NEXT_STATE_DECODE: process (state, PS2_CLK_S)
    begin
        --declare default state for next_state to avoid latches
        next_state <= state;  --default is to stay in current state

        case (state) is
            when ST_IDLE =>
                if PS2_CLK_S = '0' then
                    next_state <= ST_ACTIVE;
                end if;
            
            when ST_ACTIVE =>
                next_state <=  ST_WAIT;

            when ST_WAIT =>
                if PS2_CLK_S = '1' then
                    next_state <= ST_IDLE;
                end if;

            when others =>
                next_state <= ST_IDLE;
        end case;
    end process;
end Behavioral;
